//------------------------------------------------------------------
//
//------------------------------------------------------------------
#property indicator_chart_window
#property indicator_buffers 7
#property indicator_color1  clrDimGray
#property indicator_color2  clrDarkGray
#property indicator_color3  clrRed
#property indicator_color4  clrLimeGreen
#property indicator_color5  clrSandyBrown
#property indicator_color6  clrLimeGreen
#property indicator_color7  clrSandyBrown
#property indicator_width2  2
#property indicator_width3  2
#property indicator_style1  STYLE_DOT
#property strict


enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average,    // Average (high+low+open+close)/4
   pr_medianb,    // Average median body (open+close)/2
   pr_tbiased,    // Trend biased price
   pr_tbiased2,   // Trend biased (extreme) price
   pr_haclose,    // Heiken ashi close
   pr_haopen ,    // Heiken ashi open
   pr_hahigh,     // Heiken ashi high
   pr_halow,      // Heiken ashi low
   pr_hamedian,   // Heiken ashi median
   pr_hatypical,  // Heiken ashi typical
   pr_haweighted, // Heiken ashi weighted
   pr_haaverage,  // Heiken ashi average
   pr_hamedianb,  // Heiken ashi median body
   pr_hatbiased,  // Heiken ashi trend biased price
   pr_hatbiased2  // Heiken ashi trend biased (extreme) price
};
extern int      period   = 5;        // EMA period
extern enPrices Price    = pr_close; // Price
extern double   Factor   = 2;        // Bands deviations
extern int      LastBar  = 0;        // Last bar in the past data
extern int      PastBars = 200;      // Number of past bars
extern int      FutBars  = 50;       // Number of bars to predict 
extern int      HarmNo   = 8;        // Total number of harmonics in model
extern double   FreqTOL  = 0.0001;   // Tolerance of frequency calculations

double in[],pv[],fv[],upp[],dnp[],upf[],dnf[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   if(HarmNo==0 || HarmNo>PastBars) HarmNo=PastBars; LastBar = MathMax(LastBar,0);
      SetIndexBuffer(0,in);
      SetIndexBuffer(1,pv);  SetIndexShift(1,       -LastBar);                        // past data vector 0..np-1; 0 corresponds to bar=LastBar
      SetIndexBuffer(2,fv);  SetIndexShift(2,FutBars-LastBar);                        // future data vector i=0..nf; nf corresponds to bar=LastBar
      SetIndexBuffer(3,upp); SetIndexShift(3,       -LastBar); SetIndexLabel(3,NULL); // past data vector
      SetIndexBuffer(4,dnp); SetIndexShift(4,       -LastBar); SetIndexLabel(4,NULL); // past data vector
      SetIndexBuffer(5,upf); SetIndexShift(5,FutBars-LastBar); SetIndexLabel(5,NULL); // future data vector
      SetIndexBuffer(6,dnf); SetIndexShift(6,FutBars-LastBar); SetIndexLabel(6,NULL); // future data vector
         

   IndicatorShortName("ema extrapolatoed ("+(string)period+","+(string)PastBars+","+(string)HarmNo+")");  
   return(0);
}
int deinit(){ return(0); }

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double x[],xp[],xf[];
int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit=MathMin(Bars-counted_bars,Bars-1);

         //
         //
         //
         //
         //

         if (Bars<PastBars) return(0);
         int pastBars=MathMin(PastBars-LastBar,Bars);
         if (ArraySize(x)!=pastBars) 
         { 
             ArrayResize(x ,pastBars); 
             ArrayResize(xp,pastBars); 
         }
         int futBars=MathMin(pastBars-1,FutBars);
         if (ArraySize(xf)!=futBars+1) 
             ArrayResize(xf,futBars+1); 

         //
         //
         //
         //
         //
           
         double av=0;
             for(int i=limit;                              i>=0;       i--) in[i]=iEma(getPrice(Price,Open,Close,High,Low,i),period,i); 
             for(int i=MathMin(pastBars+LastBar-1,Bars-1); i>=LastBar; i--) av+=in[i]; av/=pastBars;
               ArrayInitialize(xp,av);
               ArrayInitialize(xf,av);
                  ArrayCopy(x,in,0,LastBar,pastBars);
   
   //
   //
   //
   //
   //
   
   double w,m,c,s,dev=0;
   for(int harm=1;harm<=HarmNo;harm++)
   {
      Freq(pastBars,FreqTOL,x,xp,w,m,c,s);
      for(int i=0; i<pastBars; i++) 
      {
                           xp[i] += m+c*cos(w*i)+s*sin(w*i);
         if (i<=futBars)   xf[i] += m+c*cos(w*i)-s*sin(w*i);
         if (harm==HarmNo) dev   += MathPow(x[i]-xp[i],2);
      }       
   }
   ArrayCopy(pv,xp); ArrayCopy(fv,xf); dev = MathSqrt(dev/(pastBars+1))*Factor;
   for(int i=0;i<pastBars;i++) 
   {
      upp[i] = pv[i] + dev;
      dnp[i] = pv[i] - dev;
      upf[i] = (i<=futBars) ? fv[i] + dev : EMPTY_VALUE;
      dnf[i] = (i<+futBars) ? fv[i] - dev : EMPTY_VALUE;
   }      
   #define _invert(_arr,_1,_2) tmp=_arr[_1]; _arr[_1]=_arr[_2-_1]; _arr[_2-_1]=tmp;
   for(int i=0;i<=(futBars-1)/2;i++)
   {
      double tmp;
          _invert(fv ,i,FutBars);
          _invert(upf,i,FutBars);
          _invert(dnf,i,FutBars);
   } 

   //
   //
   //
   //
   //

   SetIndexDrawBegin(1,Bars-PastBars);
   SetIndexDrawBegin(3,Bars-PastBars);
   SetIndexDrawBegin(4,Bars-PastBars);
   SetIndexDrawBegin(2,Bars-FutBars-1);
   SetIndexDrawBegin(5,Bars-FutBars-1);
   SetIndexDrawBegin(6,Bars-FutBars-1);
   return(0); 
}


//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workEma[][1];
double iEma(double price, double tperiod, int r, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars); r=Bars-r-1;

   //
   //
   //
   //
   //
      
   workEma[r][instanceNo] = price;
   double ;
   if (r>0 && tperiod>1)
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+(2.0/(1.0+tperiod))*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
}

//------------------------------------------------------------------
//
//    Quinn and Fernandes algorithm
//
//------------------------------------------------------------------
//
//
//
//
//
//

double _z[];
void Freq(int np, double freqTol, double& _x[], double& values[], double& w, double& m, double& c, double& s)
{
   if (ArraySize(_z)!=np) ArrayResize(_z,np);
   
   //
   //
   //
   //
   //
   
   double a=0.0, b=2.0; _z[0]=_x[0]-values[0];
   while(MathAbs(a-b)>freqTol)
   {
      a=b;
            _z[1]=_x[1]-values[1]+a*_z[0];
               double num=_z[0]*_z[1];
               double den=_z[0]*_z[0];
               for(int i=2;i<np;i++)
               {
                  _z[i] = _x[i]-values[i]+a*_z[i-1]-_z[i-2];
                     num  += _z[i-1]*(_z[i]+_z[i-2]);
                     den  += _z[i-1]*_z[i-1];
               }
               b = (den!=0) ? num/den : 0;
   }
   w=MathArccos(MathMin(MathMax(b/2.0,-1),1));

   //
   //
   //
   //
   //
      
   double Sc=0,Ss=0,Scc=0,Sss=0,Scs=0,Sx=0,Sxc=0,Sxs=0;;
      for(int i=0;i<np;i++)
      {
         double wcos = cos(w*i);
         double wsin = sin(w*i);
                Sc  += wcos;
                Ss  += wsin;
                Scc += wcos*wcos;
                Sss += wsin*wsin;
                Scs += wcos*wsin;
                Sx  += (_x[i]-values[i]);
                Sxc += (_x[i]-values[i])*wcos;
                Sxs += (_x[i]-values[i])*wsin;
      }
      Sc/=np; Ss/=np; Scc/=np; Sss/=np; Scs/=np; Sx/=np; Sxc/=np; Sxs/=np; m=Sx; c=0.0;  s=0.0;
      if(w!=0.0)
      {
         double den = MathPow(Scs-Sc*Ss,2)-(Scc-Sc*Sc)*(Sss-Ss*Ss);
                c = ((Sxs-Sx*Ss)*(Scs-Sc*Ss)-(Sxc-Sx*Sc)*(Sss-Ss*Ss))/den;
                s = ((Sxc-Sx*Sc)*(Scs-Sc*Ss)-(Sxs-Sx*Ss)*(Scc-Sc*Sc))/den;
                m = Sx-c*Sc-s*Ss;
      }
   return;
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//
//

#define priceInstances 1
double workHa[][priceInstances*4];
double getPrice(int tprice, const double& open[], const double& close[], const double& high[], const double& low[], int i, int instanceNo=0)
{
  if (tprice>=pr_haclose)
   {
      if (ArrayRange(workHa,0)!= Bars) ArrayResize(workHa,Bars); instanceNo*=4;
         int r = Bars-i-1;
         
         //
         //
         //
         //
         //
         
         double haOpen;
         if (r>0)
                haOpen  = (workHa[r-1][instanceNo+2] + workHa[r-1][instanceNo+3])/2.0;
         else   haOpen  = (open[i]+close[i])/2;
         double haClose = (open[i] + high[i] + low[i] + close[i]) / 4.0;
         double haHigh  = MathMax(high[i], MathMax(haOpen,haClose));
         double haLow   = MathMin(low[i] , MathMin(haOpen,haClose));

         if(haOpen  <haClose) { workHa[r][instanceNo+0] = haLow;  workHa[r][instanceNo+1] = haHigh; } 
         else                 { workHa[r][instanceNo+0] = haHigh; workHa[r][instanceNo+1] = haLow;  } 
                                workHa[r][instanceNo+2] = haOpen;
                                workHa[r][instanceNo+3] = haClose;
         //
         //
         //
         //
         //
         
         switch (tprice)
         {
            case pr_haclose:     return(haClose);
            case pr_haopen:      return(haOpen);
            case pr_hahigh:      return(haHigh);
            case pr_halow:       return(haLow);
            case pr_hamedian:    return((haHigh+haLow)/2.0);
            case pr_hamedianb:   return((haOpen+haClose)/2.0);
            case pr_hatypical:   return((haHigh+haLow+haClose)/3.0);
            case pr_haweighted:  return((haHigh+haLow+haClose+haClose)/4.0);
            case pr_haaverage:   return((haHigh+haLow+haClose+haOpen)/4.0);
            case pr_hatbiased:
               if (haClose>haOpen)
                     return((haHigh+haClose)/2.0);
               else  return((haLow+haClose)/2.0);        
            case pr_hatbiased2:
               if (haClose>haOpen)  return(haHigh);
               if (haClose<haOpen)  return(haLow);
                                    return(haClose);        
         }
   }
   
   //
   //
   //
   //
   //
   
   switch (tprice)
   {
      case pr_close:     return(close[i]);
      case pr_open:      return(open[i]);
      case pr_high:      return(high[i]);
      case pr_low:       return(low[i]);
      case pr_median:    return((high[i]+low[i])/2.0);
      case pr_medianb:   return((open[i]+close[i])/2.0);
      case pr_typical:   return((high[i]+low[i]+close[i])/3.0);
      case pr_weighted:  return((high[i]+low[i]+close[i]+close[i])/4.0);
      case pr_average:   return((high[i]+low[i]+close[i]+open[i])/4.0);
      case pr_tbiased:   
               if (close[i]>open[i])
                     return((high[i]+close[i])/2.0);
               else  return((low[i]+close[i])/2.0);        
      case pr_tbiased2:   
               if (close[i]>open[i]) return(high[i]);
               if (close[i]<open[i]) return(low[i]);
                                     return(close[i]);        
   }
   return(0);
}